#include "data_link_rx.h"
#include <assert.h>


void data_link_rx_init(data_link_rx_t *data_link)
{
    data_link->last_bit = false;
    data_link->l_time = 0;
    data_link->h_time = 0;
    data_link->data = 0;
    data_link->data_bits_n = 0;
    data_link->start_condition = false;
}


void data_link_rx_receive(data_link_rx_t *data_link, uint8_t wire_data)
{
    for (uint8_t bit_index = 0x80; bit_index != 0; bit_index >>= 1)
    {
        bool bit = wire_data & bit_index;

        // Detect signal edge
        if (bit != data_link->last_bit)
        {
            if (!bit) // H -> L
            {
                // Check for start-of-transmission indicator
                if (data_link->h_time * 10 <= data_link->l_time)
                {
                    data_link->data_bits_n = 0;
                    data_link->start_condition = true;
                }
                else
                {
                    if (data_link->h_time >= data_link->l_time)
                    {
                        data_link->data |= (0x8000 >> data_link->data_bits_n);
                    }

                    ++data_link->data_bits_n;
                }

                data_link->l_time = 0;
            }
            else // L -> H
            {
                data_link->h_time = 0;
            }

            data_link->last_bit = bit;
        }

        if (bit)
        {
            if (data_link->h_time < 0xffff)
            {
                ++data_link->h_time;
            }
        }
        else
        {
            if (data_link->l_time < 0xffff)
            {
                ++data_link->l_time;
            }
        }
    }
}


bool data_link_rx_get_start_condition(data_link_rx_t *data_link)
{
    bool result = data_link->start_condition;
    data_link->start_condition = false;
    return result;
}


bool data_link_rx_has_data(data_link_rx_t const *data_link)
{
    return (data_link->data_bits_n >= 8);
}


uint8_t data_link_rx_get_data(data_link_rx_t *data_link)
{
    assert(data_link_rx_has_data(data_link));

    uint8_t result = data_link->data >> 8;
    data_link->data <<= 8;
    data_link->data_bits_n -= 8;
    return result;
}